//------------------------------------------------------------------
#property copyright ""
#property link      ""
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   1
#property indicator_label1  "MA ribbon"
#property indicator_type1   DRAW_FILLING
#property indicator_color1  clrLimeGreen,clrOrange
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

//
//
//
//
//

enum enAvgType
{
   avgSma,    // Simple moving average
   avgEma,    // Exponential moving average
   avgSmma,   // Smoothed MA
   avgLwma    // Linear weighted MA
};
enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average     // Average (high+low+oprn+close)/4
};

input ENUM_TIMEFRAMES    FastTimeFrame = PERIOD_CURRENT; // Fast time frame
input int                FastMA        = 9;              // Fast MA period
input enAvgType          AvgMethodf    = avgSma;         // Fast MA method
input enPrices           Pricef        = pr_close;       // Fast MA price
input ENUM_TIMEFRAMES    SlowTimeFrame = PERIOD_CURRENT; // Slow time frame
input int                SlowMA        = 76;             // Slow MA period
input enAvgType          AvgMethods    = avgEma;         // Slow MA method
input enPrices           Prices        = pr_weighted;    // Slow MA price

//
//
//
//
//

double ma1[];
double ma2[];
string name;
int ma1handle;
int ma2handle;
ENUM_TIMEFRAMES ftime;
ENUM_TIMEFRAMES stime;
//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,ma1,INDICATOR_DATA);
   SetIndexBuffer(1,ma2,INDICATOR_DATA);
      name = getIndicatorName();
      if (FastTimeFrame!=-99)
      {
         ftime=MathMax(_Period,FastTimeFrame);
         stime=MathMax(_Period,SlowTimeFrame);
            ma1handle = iCustom(NULL,ftime,name,-99,FastMA,AvgMethodf,Pricef,0);
            ma2handle = iCustom(NULL,stime,name,-99,SlowMA,AvgMethods,Prices,0);
      }         
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int totalBars;
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{
   //
   //
   //
   //
   //

   if (FastTimeFrame==-99)
   {
      totalBars = rates_total;
      for (int i=(int)MathMax(prev_calculated-1,1); i<rates_total; i++)
         ma1[i] = iCustomMa(AvgMethodf,getPrice(Pricef,open,close,high,low,rates_total,i),FastMA,i,0);
         ma2[rates_total-1] = MathMax(rates_total-prev_calculated+1,1);
      return(rates_total);
   }
   
   //
   //
   //
   //
   //
         
      datetime timef[]; 
      datetime startTime = time[0]-PeriodSeconds(ftime);
      datetime endTime   = time[rates_total-1];
         int bars = CopyTime(NULL,ftime,startTime,endTime,timef);
            if (timef[0]>time[0] || bars<1) return(prev_calculated);
               double valuef[]; CopyBuffer(ma1handle,0,0,bars,valuef);
               double countf[]; CopyBuffer(ma1handle,1,0,bars,countf);
            int maxb = (int)MathMax(MathMin(countf[bars-1]*PeriodSeconds(ftime)/PeriodSeconds(Period()),rates_total-1),1);
            
            
         //
         //
         //
         //
         //
                     
         for(int i=(int)MathMax(prev_calculated-maxb,0); i<rates_total; i++)
         {
            int d = dateArrayBsearch(timef,time[i],bars);
            if (d > -1 && d < bars)
               ma1[i] = valuef[d];
         
            //
            //
            //
            //
            //
         
            int l=MathMin(i+1,rates_total-1);
            if (d!=dateArrayBsearch(timef,time[l],bars) || i==l)
            {
               int n,k;
                  for(n = 1; (i-n)> 0 && time[i-n] >= timef[d]; n++) continue;	
                  for(k = 1; (i-k)>=0 && k<n; k++)
                     ma1[i-k] = ma1[i] + (ma1[i-n]-ma1[i])*k/n;
            }
         }            

         //
         //
         //
         //
         //
         
         datetime times[]; 
            startTime = time[0]-PeriodSeconds(stime);
            endTime   = time[rates_total-1];
            bars      = CopyTime(NULL,stime,startTime,endTime,times);
            if (times[0]>time[0] || bars<1) return(prev_calculated);
                  double values[]; CopyBuffer(ma2handle,0,0,bars,values);
                  double counts[]; CopyBuffer(ma2handle,1,0,bars,counts);
            maxb = (int)MathMax(MathMin(counts[bars-1]*PeriodSeconds(stime)/PeriodSeconds(Period()),rates_total-1),1);

         //
         //
         //
         //
         //
         
         for(int i=(int)MathMax(prev_calculated-maxb,0); i<rates_total; i++)
         {
            int d = dateArrayBsearch(times,time[i],bars);
            if (d > -1 && d < bars)
               ma2[i] = values[d];

         
            //
            //
            //
            //
            //
         
            int l=MathMin(i+1,rates_total-1);
            if (d!=dateArrayBsearch(times,time[l],bars) || i==l)
            {
               int n,k;
                  for(n = 1; (i-n)> 0 && time[i-n] >= times[d]; n++) continue;	
                  for(k = 1; (i-k)>=0 && k<n; k++)
                     ma2[i-k] = ma2[i] + (ma2[i-n]-ma2[i])*k/n;
            }
         }            
   return(rates_total);
}


  
//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

#define _maWorkBufferx1 1
#define _maWorkBufferx2 2
#define _maWorkBufferx3 3

double iCustomMa(int mode, double price, double length, int r, int instanceNo=0)
{
   switch (mode)
   {
      case avgSma   : return(iSma(price,(int)length,r,instanceNo));
      case avgEma   : return(iEma(price,length,r,instanceNo));
      case avgSmma  : return(iSmma(price,(int)length,r,instanceNo));
      case avgLwma  : return(iLwma(price,(int)length,r,instanceNo));
      default       : return(price);
   }
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workSma[][_maWorkBufferx2];
double iSma(double price, int period, int r, int instanceNo=0)
{
   if (ArrayRange(workSma,0)!= totalBars) ArrayResize(workSma,totalBars); instanceNo *= 2;

   //
   //
   //
   //
   //

   int k;      
      workSma[r][instanceNo]    = price;
      workSma[r][instanceNo+1]  = 0; for(k=0; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo];  
      workSma[r][instanceNo+1] /= (double)k;
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][_maWorkBufferx1];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= totalBars) ArrayResize(workEma,totalBars);

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

double workSmma[][_maWorkBufferx1];
double iSmma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= totalBars) ArrayResize(workSmma,totalBars);

   //
   //
   //
   //
   //

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workLwma[][_maWorkBufferx1];
double iLwma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= totalBars) ArrayResize(workLwma,totalBars);
   
   //
   //
   //
   //
   //
   
   workLwma[r][instanceNo] = price;
      double sumw = period;
      double sum  = period*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}


//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double getPrice(enPrices price, const double& open[], const double& close[], const double& high[], const double& low[], int bars, int i)
{
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int dateArrayBsearch(datetime& times[], datetime toFind, int total)
{
   int mid   = 0;
   int first = 0;
   int last  = total-1;
   
   while (last >= first)
   {
      mid = (first + last) >> 1;
      if (toFind == times[mid] || (mid < (total-1) && (toFind > times[mid]) && (toFind < times[mid+1]))) break;
      if (toFind <  times[mid])
            last  = mid - 1;
      else  first = mid + 1;
   }
   return (mid);
}

//
//
//
//
//

string getIndicatorName()
{
   string progPath    = MQL5InfoString(MQL5_PROGRAM_PATH);
   string toFind      = "MQL5\\Indicators\\";
   int    startLength = StringFind(progPath,toFind)+StringLen(toFind);
         
         string indicatorName = StringSubstr(progPath,startLength);
                indicatorName = StringSubstr(indicatorName,0,StringLen(indicatorName)-4);
   return(indicatorName);
}

//
//
//
//
//

string periodToString(int period)
{
   int i;
   static int    _per[]={1,2,3,4,5,6,10,12,15,20,30,0x4001,0x4002,0x4003,0x4004,0x4006,0x4008,0x400c,0x4018,0x8001,0xc001};
   static string _tfs[]={"1 minute","2 minutes","3 minutes","4 minutes","5 minutes","6 minutes","10 minutes","12 minutes",
                         "15 minutes","20 minutes","30 minutes","1 hour","2 hours","3 hours","4 hours","6 hours","8 hours",
                         "12 hours","daily","weekly","monthly"};
   
   if (period==PERIOD_CURRENT) 
       period = Period();   
            for(i=0;i<20;i++) if(period==_per[i]) break;
   return(_tfs[i]);   
}